//+------------------------------------------------------------------+
//|                                                 Wilder's DMI.mq4 |
//|                                                                  |
//| Directional movement index was developed by Welles Wilder        |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "www.forex-tsd.com"

#property indicator_separate_window
#property  indicator_buffers 5
#property indicator_minimum  0
#property indicator_color1   Lime
#property indicator_color2   Tomato
#property indicator_color3   DimGray
#property indicator_color4   Orange
#property indicator_color5   DimGray
#property indicator_width3   2 
#property indicator_width4   2 
#property indicator_style1   STYLE_DOT
#property indicator_style2   STYLE_DOT
#property indicator_style5   STYLE_DOT

//
//
//
//
//

extern int  DMI.Length        =    14;
extern bool ShowADX           =  true;
extern bool ShowADXR          = false;
extern int  Level             =    20;
extern bool alertsOn          = false;
extern bool alertsOnCurrent   = false;
extern bool alertsOnCrossDown = false;
extern bool alertsOnCrossUp   = true;
extern bool alertsMessage     = true;
extern bool alertsSound       = false;
extern bool alertsEmail       = false;

//
//
//
//
//

double DIp[];
double DIm[];
double ADX[];
double ADXR[];
double ADXLevel[];
double averageDIp[];
double averageDIm[];
double averageTR[];


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//

int init()
{
   IndicatorBuffers(8);
   SetIndexBuffer(0,DIp);      SetIndexLabel(0,"DI+");
   SetIndexBuffer(1,DIm);      SetIndexLabel(1,"DI-");
   SetIndexBuffer(2,ADX);      SetIndexLabel(2,"ADX");
   SetIndexBuffer(3,ADXR);     SetIndexLabel(3,"ADXR");
   SetIndexBuffer(4,ADXLevel); SetIndexLabel(4,NULL);
   SetIndexBuffer(5,averageDIp);
   SetIndexBuffer(6,averageDIm);
   SetIndexBuffer(7,averageTR);
      for (int i=0;i<8;i++) SetIndexEmptyValue(i,0.00);

   //
   //
   //
   //
   //

   IndicatorShortName("Wilder\'s DMI ("+DMI.Length+")");
   return(0);
}

int deinit()
{
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double trend[][2];
#define _dmim 0
#define _dmip 1

int start()
{
   int i,r,counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-2);
           if (ArrayRange(trend,0)!=Bars) ArrayResize(trend,Bars);

   //
   //
   //
   //
   //
   
   double sf = (DMI.Length-1.0)/DMI.Length;
      for (i=limit, r=Bars-i-1; i>=0; i--,r++)
      {
         double currTR  = MathMax(High[i],Close[i+1])-MathMin(Low[i],Close[i+1]);
         double DeltaHi = High[i] - High[i+1];
	      double DeltaLo = Low[i+1] - Low[i];
         double plusDM  = 0.00;
         double minusDM = 0.00;
         
            if ((DeltaHi > DeltaLo) && (DeltaHi > 0)) plusDM  = DeltaHi;
            if ((DeltaLo > DeltaHi) && (DeltaLo > 0)) minusDM = DeltaLo;      
         
         //
         //
         //
         //
         //
         
            averageDIp[i] = sf*averageDIp[i+1] + plusDM;
            averageDIm[i] = sf*averageDIm[i+1] + minusDM;
            averageTR[i]  = sf*averageTR[i+1]  + currTR;
            ADXLevel[i]   = Level;

         //
         //
         //
         //
         //
                  
            DIp[i] = 0.00;                   
            DIm[i] = 0.00;                   
            if (averageTR[i] > 0)
               {              
                  DIp[i] = 100.00 * averageDIp[i]/averageTR[i];
                  DIm[i] = 100.00 * averageDIm[i]/averageTR[i];
               }            

            if(ShowADX)
               {
                  double DX;
                  if((DIp[i] + DIm[i])>0) 
                       DX = 100*MathAbs(DIp[i] - DIm[i])/(DIp[i] + DIm[i]); 
                  else DX = 0.00;
                  ADX[i] = sf*ADX[i+1] + DX/DMI.Length; 
                  if(ShowADXR)
                         ADXR[i] = 0.5*(ADX[i] + ADX[i+DMI.Length]);
               }
         trend[r][_dmim] = trend[r-1][_dmim];
         trend[r][_dmip] = trend[r-1][_dmip];
            if (DIp[i]>ADX[i]) trend[r][_dmip] =  1;
            if (DIp[i]<ADX[i]) trend[r][_dmip] = -1;
            if (DIm[i]>ADX[i]) trend[r][_dmim] =  1;
            if (DIm[i]<ADX[i]) trend[r][_dmim] = -1;
      }   
   manageAlerts();      
   return(0);      
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = Bars-whichBar-1;
      
      //
      //
      //
      //
      //
      
      static datetime prevTime1  = 0;
      static string   prevAlert1 = "";
         if (trend[whichBar][_dmip] != trend[whichBar-1][_dmip])
         {
            if (alertsOnCrossUp   && trend[whichBar][_dmip] ==  1) doAlert(prevTime1,prevAlert1," DMI+ crossed ADX up");
            if (alertsOnCrossDown && trend[whichBar][_dmip] == -1) doAlert(prevTime1,prevAlert1," DMI+ crossed ADX down");
         }
      static datetime prevTime2  = 0;
      static string   prevAlert2 = "";
         if (trend[whichBar][_dmim] != trend[whichBar-1][_dmim])
         {
            if (alertsOnCrossUp   && trend[whichBar][_dmim] ==  1) doAlert(prevTime2,prevAlert2," DMI- crossed ADX up");
            if (alertsOnCrossDown && trend[whichBar][_dmim] == -1) doAlert(prevTime2,prevAlert2," DMI- crossed ADX down");
         }
   }
}

//
//
//
//
//

void doAlert(datetime& previousTime, string& previousAlert, string doWhat)
{
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[0]) {
       previousAlert  = doWhat;
       previousTime   = Time[0];

       //
       //
       //
       //
       //

       message =  Symbol()+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+doWhat;
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol(),"Wilder\'s DMI "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

